// emacs: this is -*- c++ -*-
//
//   @file    fastnlo_utils.h
//
//   $Id: fastnlo_utils.h, v0.0   Mon 25 Apr 2016 15:03:24 CEST sutt $


#ifndef  FASTNLO_UTILS_H
#define  FASTNLO_UTILS_H

#ifdef HAVE_FASTNLO

#include "amconfig.h"

#include "fastnlotk/fastNLOCreate.h"

namespace nnlo {

   void init_fastnlo( const int& id, const std::string& gridname, const int& nbins, const double& lo );

   void fill_fastnlo( const int& id, const double& obs, const double* wt, const double* wtref );

   void term_fastnlo( const int& id, int nweights );

   void fill_ref_fastnlo( const int& id, const double& obs, const double* wt );

}

namespace fnloUtils {

   //
   // --- global helper functions for NNLOJET interface
   //
   fastNLO::GeneratorConstants  GetNNLOJET_GenConsts();
   fastNLO::ProcessConstants    GetNNLOJET_ProcConstsPP();
   fastNLO::ProcessConstants    GetNNLOJET_ProcConstsDIS();
   fastNLO::ScenarioConstants   GetNNLOJET_ScenConsts();
   void  InitFastNLO(const int& id, const double* wt);
   //std::string  GetProcessName();
   //inline bool  IsDIS() {static bool bIsDIS = (GetProcessName() == "DIS"); return bIsDIS;};
   std::string  GetOrderName(std::set<std::pair<int,int> >* nlne = NULL );
   //int          GetOrder();
   std::string  GetContribName(int nloops=-1, int ne = -1);
   std::string  GetConfigFile();
   std::vector<std::string>  GetConfigFiles();
   std::string  GetConfigDir();
   int          GetSeed();
   void         RememberContribution();
   double       CalculateReferenceWeight(double w, double x1, double x2);
   std::vector<double>   SclIndepWgt(const double* w, double muf, double mur);
   void         SetNNLOJETDefaultBinning(fastNLO::ScenarioConstants& sc, const int& nbins, const double& lo);
   template <typename T>
   std::vector<std::vector<std::pair<T,T> > > VectorToPairs(const std::vector<std::vector<T> >& v) {
      //!< convert a vector of vectors to a vector of pairs
      std::vector<std::vector<std::pair<T,T> > > out(v.size());
      for ( unsigned int ip = 0 ;ip<v.size() ; ip++ ) {
         //if ( v[ip].size()%2 == 1 ) std::cout<<"[fnloUtils::VectorToPairs] ERROR! even number of integers required!"<<std::endl;
         // for ( unsigned int p = 0; p<v[ip].size() ; p+=2 ) {
         if ( v[ip].size()%2 == 0 ) std::cout<<"[fnloUtils::VectorToPairs] ERROR! Odd number of integers required (because first values is just a counter)!"<<std::endl;
         for ( unsigned int p = 1; p<v[ip].size() ; p+=2 ) {
            out[ip].push_back(std::make_pair(v[ip][p],v[ip][p+1]));
         }
      }
      return out;
   }

   // class to handle the sub processes
   class fnloSuprocesses : public std::vector<int> {
   public:
      fnloSuprocesses(const std::string& config) ; //!< constructor reading config file with subprocesses
      fnloSuprocesses(){} ; //!< default constructor (don't call ist)
      void ReadConfigFile(const std::string& config) ; //!< read config file
      void ReadConfigFiles(const vector<std::string>& config) { for ( const auto& i : config ) ReadConfigFile(i);} ; //!< read config file
      std::vector<std::vector<int > >  GetPartonCombinations() const; //!< get parton combinations (fUniqueValues)
      const std::vector<std::vector<int > >&  GetInitialValues() const { return fInitialValues;}
      const std::vector<std::vector<int > >&  GetUniqueValues() const { return fUniqueValues;}
      const std::string& GetConfigFile() const {return fConfigFile;}
   private:
      std::vector<std::vector<int > > fInitialValues;
      std::vector<std::vector<int > > fUniqueValues;
      std::string fConfigFile;
      bool CompareVectors(const std::vector<int>& v1, const std::vector<int>& v2 ) const;
      void SetUniqueValues();
   };

   // class to remember the 'init' varialbes
   struct fnloInit {
      int id;
      std::string gridname;
      int nbins;
      const double* lo;
      //const double* hi;
   };

   //
   // --- global variables
   //
   static std::vector < fastNLOCreate* > ftable;
   static std::vector < bool > fIsInclJet;
   static std::vector < bool > fIs820GeV; 
   static std::vector < bool > fIs137; 
   static std::vector < std::string > ftablename;
   static std::vector < fnloInit > fInits;
   static fnloUtils::fnloSuprocesses sp;
   static std::set<std::pair<int,int> > fNlNe; //!< keep track of what we have calculated
   static std::set<int > fIDs; //!< keep track of what we have calculated

   static double _myfillwgt = 0;

}

#endif

#endif  // FASTNLO_UTILS_H
