// is -*- c++ -*-
//
//   @file    nnlo_common.h
//            nnlo common block interface
//
//
//   Copyright (C) 2015 M.Sutton (sutt@cern.ch)
//
//   $Id: nnlo_common.h, v0.0   Thu  3 Sep 2015 09:51:20 CEST sutt $

#ifndef  NNLO_COMMON_H
#define  NNLO_COMMON_H

// #include <omp.h>

const int maxscl = 7;
//extern "C" int maxscl_hook;

const double TINY = 1.e-100;

typedef struct {
  char sproc[5];
} __process__ ;


/// vegas iteration information
typedef struct {
  int itmax1;
  int itmax2;
  int nevents;
} __runinfo__;


typedef struct { int ieorder; } __order__;


/// alpha_s
typedef struct { double as; } __strong__;

/// beam energy ????
/// typedef struct { double Ebeam; } __Ebeam__;



typedef struct {
  int njets;
   int nloop;// NOT IMPLEMENTED IN NNLOJET!
  // int norder;
} __inppar__ ;



typedef struct {
  int ndim;
  int nv;
  int norder;
} __ndimcurrent__ ;


typedef struct {
   double amzZ;
   double zewfac[4];
   double zewnull[4];
} __eweakz__ ;

typedef struct {
  double PDF;
  double xi1;
  double xi2;
  double xjac;
  double x1;
  double x2;
} __PDF__ ;


/// standard x variables
typedef struct {
  double x1;
  double x2;
} __parfrac__ ;


/// virtual correction x variables
typedef struct {
  double xreg1;
  double xreg2;
  bool   virtsflag;
} __xregions__ ;

/// jacobian flag for virtual terms
typedef struct {
  bool jflag;
} __jacobian__ ;

typedef struct {
  int igx;
} __gridix__ ;


///// scales
//typedef struct {
//  double facscale[maxscl];
//  double rscale[maxscl];
//} __scales__;


/// index of which diagram is being
/// processed ...
typedef struct {
  int iproc;
} __currentprocess__;


/// which two input parton species are
/// related to process iproc
typedef struct {
  int ip1;
  int ip2;
} __idparton__;

typedef struct {
  int ip1;
  int ip2;
} __partid__;



/// normalisation term
typedef struct {
  double rnormcurr;
  double wgtcurr;
} __histonorm__;


typedef struct {
  double gridwgt[maxscl];
} __gridweights__ ;


//   common /channel/ip(1:2000),ipars(1:2000),iloops(1:2000)


typedef struct {
  int ip[2000];
  int ipars[2000];
  int iloops[2000];
} __channel__ ;


/// jacobian factor for colour sampler
typedef struct {
  double xcoljac;
} __xcoljac__ ;

/// jacobian flag for colour sampler
typedef struct {
  bool colflag;
} __colourflag__ ;



typedef struct {
  int Nphase;
} __nphase__ ;

typedef struct {
  int iphase;
} __iphase__ ;



typedef struct {
  double wgt;
  int iave;
  int it;
  double swgt;
} __vegasiterweight__ ;

typedef struct {
  double roots;
} __inputpar__ ;


typedef struct {
   char srunid[3];
} __runid__;

typedef struct {
   char fname[100];
} __outfile__;

typedef struct {
   char sjobname[10];
} __jobname__;

typedef struct {
   char slogname[100];
} __log__;


extern "C" __runinfo__   runinfo_;
extern "C" __order__     order_;
// extern "C" __Ebeam__  ebeam_;

extern "C" __inppar__    inppar_;

extern "C" __strong__    strong_;
// extern "C" __PDF__       pdf_;

extern "C" __parfrac__   parfrac_;
#pragma omp threadprivate(parfrac_)

extern "C" __xregions__ xregions_;

extern "C" __jacobian__ jacobian_;

//extern "C" __scales__    scales_;
//#pragma omp threadprivate(scales_)

extern "C" double mur2_hook(const int& idx);
extern "C" double muf2_hook(const int& idx);
extern "C" int maxscl_hook; //!< number of requested scale variations
extern "C" int iseed_hook; //!< seed
extern "C" int obs_id_hook(const char obsname[]); //!< observable id
extern "C" double obs_val_hook(const int& obsid); //!< observable value

extern "C" __histonorm__ histonorm_;

extern "C" __currentprocess__ currentprocess_;
extern "C" __idparton__       idparton_;

extern "C" __partid__         partid_;
#pragma omp threadprivate(partid_)

// extern "C" __gridweights__    gridweights_;

extern "C" __ndimcurrent__   ndimcurrent_;
extern "C" __eweakz__        eweakz_;


extern "C" __gridix__      gridix_;

extern "C" __channel__     channel_;

extern "C"  __process__    process_;

extern "C" __colourflag__  colourflag_;

extern "C" __xcoljac__     xcoljac_;

/// access the rotated phasespace points
/// for the real caching
extern "C" __nphase__   nphase_;

extern "C" __iphase__   iphase_;

extern "C" __vegasiterweight__ vegasiterweight_;
extern "C" __runid__ runid_;
extern "C" __outfile__ outfile_;
extern "C" __jobname__ jobname_;
extern "C" __inputpar__ inputpar_;
extern "C" __log__ log_;

#endif  // NNLO_COMMON_H
